Package: systemfit
Version: 1.1-30
Date: 2023-03-22
Title: Estimating Systems of Simultaneous Equations
Author: Arne Henningsen and Jeff D. Hamann
Maintainer: Arne Henningsen <arne.henningsen@gmail.com>
Depends: R (>= 3.2.0), Matrix, car (>= 2.0-0), lmtest
Suggests: knitr, plm (>= 1.0-1), sem (>= 2.0-0)
Imports: stats (>= 2.14.0), sandwich (>= 2.2-9), MASS, methods
Description: Econometric estimation of simultaneous
 systems of linear and nonlinear equations using Ordinary Least
 Squares (OLS), Weighted Least Squares (WLS), Seemingly Unrelated
 Regressions (SUR), Two-Stage Least Squares (2SLS), Weighted
 Two-Stage Least Squares (W2SLS), and Three-Stage Least Squares (3SLS)
 as suggested, e.g., by Zellner (1962) <doi:10.2307/2281644>,
 Zellner and Theil (1962) <doi:10.2307/1911287>, and
 Schmidt (1990) <doi:10.1016/0304-4076(90)90127-F>.
License: GPL (>= 2)
URL: https://r-forge.r-project.org/projects/systemfit/
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-03-22 13:15:35 UTC; gsl324
Repository: CRAN
Date/Publication: 2023-03-22 17:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:06:55 UTC; unix
